/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * Projektbeschreibung:
 * @title
 * @version 1.0
 * @copyright Copyright (c) 2004
 */

/**
 * History:
 *
 * $Log: Strasse.java,v $
 * Revision 1.2  2007/08/22 07:48:43  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:44:14  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.2  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.1  2005/01/21 13:26:08  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:19  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp straße
 */
public class Strasse extends ConfigurationObjectWrapper {

	private static final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger LOGGER = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
			.getLoggerFactory().getLogger();

	private static WeakHashMap<String, Strasse> __Cache = new WeakHashMap<>();
	private static ConfigurationObjectType __ConfigurationObjectType;

	private String _Typ;
	private long _Nummer;
	private String _Zusatz;

	private static AttributeGroup __AtgStrasse;

	public String getTyp() {
		return _Typ;
	}

	public long getNummer() {
		return _Nummer;
	}

	public String getZusatz() {
		return _Zusatz;
	}

	public Strasse(final String pid) throws DataModellObjectNotFoundException, DataModellMismatchException {
		super(pid);
	}

	private Strasse(final ConfigurationObject confObj) throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject confObj) throws DataModellMismatchException {

		final Data oData = confObj.getConfigurationData(Strasse.__AtgStrasse);
		if (oData != null) {
			if (oData.getUnscaledValue("Nummer").isNumber()) {
				try {
					_Nummer = oData.getUnscaledValue("Nummer").longValue();
				} catch (final Exception e) {
					e.printStackTrace();
				}
			}
			if (oData.getUnscaledValue("Typ").isState()) {
				_Typ = oData.getUnscaledValue("Typ").getState().getName();
			}
			_Zusatz = oData.getTextValue("Zusatz").getText();
		} else {
			Strasse.LOGGER.warn("Keine atg.straße für Object: " + getPID() + " vorhanden.");
		}

		Strasse.__Cache.put(this.getPID(), this);

	}

	/**
	 * gibt die Instance der Strasse mit der PID zurück. Es wird eine
	 * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden ist
	 * oder das zur PID gehörende Objekt nicht vom typ straße ist.
	 *
	 * @param pid
	 *            String
	 * @throws DataModellObjectNotFoundException
	 *             das angeforderte Objekt wurde nicht gefunden
	 * @throws DataModellMismatchException
	 *             das Datenmodell ist ungültig
	 * @return Strasse
	 */
	public static Strasse getInstance(final String pid)
			throws DataModellObjectNotFoundException, DataModellMismatchException {

		Strasse retValue = Strasse.__Cache.get(pid);
		if (retValue == null) {
			retValue = new Strasse(pid);
		}
		return retValue;
	}

	/**
	 * gibt die DatenModellObjekte vom typ straße zurück
	 *
	 * @throws DataModellMismatchException
	 *             das Detanmodell ist ungültig
	 * @return Strasse[]
	 */
	public static Strasse[] getAll() throws DataModellMismatchException {

		final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
		final List<SystemObject> list = model.getType("typ.straße").getElements();
		final Strasse[] retValue = new Strasse[list.size()];
		for (int i = 0; i < list.size(); i++) {
			retValue[i] = new Strasse((ConfigurationObject) list.get(i));
		}
		return retValue;
	}

	@Override
	ConfigurationObjectType getType() {
		if (Strasse.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			Strasse.__ConfigurationObjectType = (ConfigurationObjectType) model.getType("typ.straße");
			Strasse.__AtgStrasse = model.getAttributeGroup("atg.straße");
			final List<SystemObject> list = Strasse.__ConfigurationObjectType.getObjects();
			model.getConfigurationData(list, Strasse.__AtgStrasse);
		}
		return Strasse.__ConfigurationObjectType;
	}

	@Override
	public String getTreeString() {
		return getName();
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("Strasse -- PID: \"").append(getPID());
		sb.append("\"  Name: \"").append(getName());
		sb.append("\"  Nummer: \"").append(_Nummer);
		sb.append("\"  Typ: \"").append(_Typ);
		sb.append("\"  Zusatz: \"").append(_Zusatz);
		return sb.toString();
	}

}
