/**
 * Segment 4 Datenübernahme und Aufbereitung (DUA), SWE 4.10 Ergänzung BASt-Band
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.bastband.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;
import javax.swing.tree.DefaultMutableTreeNode;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;
import de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.BastExporter;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Hauptklasse des Moduls, Bearbeitet die Aufrufparameter und je nach dem
 * oeffnet eine GUI Schnittschtelle, exportiert die Daten
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class BastBand {

	private static final Debug LOGGER = Debug.getLogger();
	/**
	 * Aufrufparameter
	 */
	static private final String PAR_GUIBASTBAND = "-guiBAStBand";
	static private final String PAR_ZEITBEREICH = "-zeitBereich";
	static private final String PAR_KONFBERIECHPID = "-KonfigurationsBereichsPid";
	static private final String PAR_AUSGABEGERAET = "-ausgabeGeraet";
	static private final String PAR_BASTFORMAT = "-bastformat";
	static private final String PAR_UEBERSCHREIBEN = "-ueberschreiben";

	/**
	 * Typen fuer auslesen der Konfiguration
	 */
	static private final String TYP_KONFIGURATIONSBEREICH = "typ.konfigurationsBereich";
	static private final String TYP_LANGZEITZAEHLSTELLE = "typ.langZeitZählstelle";
	static private final String ATG_LZZ = "atg.langZeitZählstelle";

	/**
	 * Diese Klasse versendet Betriebsmeldungen
	 */
	private final MessageSender nachrichtenSender = MessageSender.getInstance();
	/**
	 * Verbindung zum DAV
	 */
	private ClientDavConnection oClientDavConnection = null;
	/**
	 * Exporter Modul
	 */
	private BastExporter bastExporter = null;
	/**
	 * Konvertierung des angegebenes Datums
	 */
	private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("MM.yyyy");
	/**
	 * Liste der LZZ und KB, die als parameter eingegeben wurden
	 */
	private String[] listLZZundKBparameter = null;
	/**
	 * Menge der exportierenden LZZ
	 */
	private final Collection<String> listLZZ = new HashSet<>();
	/**
	 * Menge der exportierenden KB
	 */
	private final Collection<String> listKB = new HashSet<>();
	/**
	 * Zeitbereich aus dem man exportieren soll
	 */
	private final Calendar calendarStart = Calendar.getInstance();
	private Calendar calendarEnd = Calendar.getInstance();
	/**
	 * Der Format des Export files
	 */
	private int BAStFormat = BastExporter.BAST_FORMAT_V2004;
	/**
	 * Export Verzeichnis
	 */
	private File ausgabeVerzeichnis = null;
	/**
	 * Ob gui gestaret werden soll
	 */
	private boolean starteGUI = false;
	/**
	 * Ob man mit dem DAV verbunden ist
	 */
	private boolean verbunden = false;
	/**
	 * Ob man den Export abbrechen soll
	 */
	private boolean abbrechen = false;
	/**
	 * Ob man die existierende Daten ueberschreiben soll
	 */
	private boolean ueberschreiben = false;

	/**
	 * Standard Konstruktor
	 */
	public BastBand() {
		calendarEnd.set(2006, 0, 1, 0, 0, 0); // ACHTUNG Januar=0 !
		calendarStart.set(2006, 0, 1, 0, 0, 0); // ACHTUNG Januar=0 !
		listLZZundKBparameter = new String[0];
		ausgabeVerzeichnis = new File("C:\\");
	}

	/**
	 * Main Methode
	 *
	 * @param args
	 *            Aufrufparameter
	 */
	public static void main(final String[] args) {
		final ArgumentList argList = new ArgumentList(args);
		final BastBand bastBand = new BastBand();
		bastBand.parseArguments(argList);
		bastBand.work(argList);

	}

	/**
	 * Verarbeitung der Argumenten.
	 *
	 * @param argumente
	 *            die Liste der Argumente
	 */
	public void parseArguments(final ArgumentList argumente) {

		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
			@Override
			public void uncaughtException(final Thread t, final Throwable e) {
				BastBand.LOGGER.error("Applikation wird wegen" + " unerwartetem Fehler beendet", e);
				e.printStackTrace();
				Runtime.getRuntime().exit(-1);
			}
		});

		if (argumente.hasArgument(BastBand.PAR_GUIBASTBAND)) {
			final Argument argParameter = argumente.fetchArgument(BastBand.PAR_GUIBASTBAND);
			if (!argParameter.hasValue()) {
				starteGUI = true;
			} else {
				starteGUI = Boolean.parseBoolean(argParameter.asString());
			}
		}
		if (!starteGUI) {
			if (!argumente.hasArgument(BastBand.PAR_AUSGABEGERAET)) {
				System.out.println("Ausgabegerät muss angegeben werden\n");
				BastBand.printVerwendung();
				System.exit(-1);
			}
			if (!argumente.hasArgument(BastBand.PAR_ZEITBEREICH)) {
				System.out.println("Zeitbereich muss angegeben werden\n");
				BastBand.printVerwendung();
				System.exit(-1);
			}
			if (!argumente.hasArgument(BastBand.PAR_KONFBERIECHPID)) {
				System.out.println("Konfigurationsbereich muss angegeben werden\n");
				BastBand.printVerwendung();
				System.exit(-1);
			}
		}

		if (argumente.hasArgument(BastBand.PAR_AUSGABEGERAET)) {
			try {
				ausgabeVerzeichnis = argumente.fetchArgument(BastBand.PAR_AUSGABEGERAET).asDirectory();
			} catch (final IllegalArgumentException e) {
				System.out.println(argumente.fetchArgument(BastBand.PAR_AUSGABEGERAET).asString()
						+ " ist kein gültiges Ausgabegerät");
				System.out.println(e.getMessage());
				BastBand.printVerwendung();
				System.exit(-1);
			}
		}
		if (argumente.hasArgument(BastBand.PAR_ZEITBEREICH)) {
			try {
				final String[] zeit = argumente.fetchArgument(BastBand.PAR_ZEITBEREICH).asString().split(":");
				if ((zeit.length > 2) || (zeit.length == 0)) {
					throw new IllegalArgumentException("Zeitbereich nicht korrekt eingegeben");
				}

				calendarStart.setTime(BastBand.DATE_FORMAT.parse(zeit[0]));

				if (zeit.length == 2) {
					calendarEnd.setTime(BastBand.DATE_FORMAT.parse(zeit[1]));
				} else {
					calendarEnd = Calendar.getInstance();
				}

				if (calendarEnd.before(calendarStart)) {
					throw new IllegalArgumentException("Zeitbereich ist negativ");
				}
			} catch (final IllegalArgumentException e) {
				System.out.println(argumente.fetchArgument(BastBand.PAR_ZEITBEREICH).asString()
						+ " ist kein gültiger Zeitbereich");
				System.out.println(e.getMessage());
				BastBand.printVerwendung();
				System.exit(-1);
			} catch (final java.text.ParseException e) {
				System.out.println(argumente.fetchArgument(BastBand.PAR_ZEITBEREICH).asString()
						+ " ist kein gültiger Zeitbereich");
				System.out.println(e.getMessage());
				BastBand.printVerwendung();
				System.exit(-1);
			}
		}

		if (argumente.hasArgument(BastBand.PAR_BASTFORMAT)) {
			final String format = argumente.fetchArgument(BastBand.PAR_BASTFORMAT).getValue();
			final int wert = BastFormat.getIntVonKommandoParam(format);
			if (wert == -1) {
				System.out.println(format + " ist kein gültiges BASt-Format");
				BastBand.printVerwendung();
				System.exit(-1);
			} else {
				BAStFormat = wert;
			}
		}
		if (argumente.hasArgument(BastBand.PAR_KONFBERIECHPID)) {
			try {
				listLZZundKBparameter = argumente.fetchArgument(BastBand.PAR_KONFBERIECHPID).getValue().split(",");
			} catch (final IllegalArgumentException e) {
				System.out.println(argumente.fetchArgument(BastBand.PAR_KONFBERIECHPID).asString()
						+ " ist kein gültiger Konfigurationsbereich");
				System.out.println(e.getMessage());
				BastBand.printVerwendung();
				System.exit(-1);
			}
		}
		if (argumente.hasArgument(BastBand.PAR_UEBERSCHREIBEN)) {
			final Argument argParameter = argumente.fetchArgument(BastBand.PAR_UEBERSCHREIBEN);
			if (!argParameter.hasValue()) {
				ueberschreiben = true;
			} else {
				ueberschreiben = Boolean.parseBoolean(argParameter.asString());
			}
		}
	}

	/**
	 * Die arbeiter Methode.
	 *
	 * @param args
	 *            die Liste der Argumente
	 */
	public void work(final ArgumentList args) {

		try {
			final ClientDavParameters parameters = new ClientDavParameters(args);
			oClientDavConnection = new LogosClientDavConnection(parameters);
			oClientDavConnection.connect();
			oClientDavConnection.login();
			verbunden = true;
		} catch (final Exception e) {
			BastBand.LOGGER.error("Fehler beim Verbinden mit Datenverteiler. Programm wird beendet.");
			System.exit(1);
		}

		bearbeiteEingabeObjektePids();

		if (starteGUI) {
			final BastBandGUI guiWnd = new BastBandGUI();
			guiWnd.setAnfangJahr(new Integer(calendarStart.get(Calendar.YEAR)));
			guiWnd.setAnfangMonat(calendarStart.get(Calendar.MONTH));
			guiWnd.setEndeJahr(new Integer(calendarEnd.get(Calendar.YEAR)));
			guiWnd.setEndeMonat(calendarEnd.get(Calendar.MONTH));
			guiWnd.setAusgabePfad(ausgabeVerzeichnis.getAbsolutePath());
			guiWnd.setZaehlstellenBaumHauptKnoten(generiereLZZBaum());
			guiWnd.setBastFormat(BastFormat.getNameVonInt(BAStFormat));
			guiWnd.setUeberSchreiben(ueberschreiben);
			guiWnd.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

			guiWnd.addActionListenerBtnAbbrechen(new ActionListener() {
				@Override
				public void actionPerformed(final ActionEvent e) {
					disconnect(false, "");
					guiWnd.setVisible(false);
					guiWnd.dispose();
				}
			});

			guiWnd.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(final WindowEvent we) {
					disconnect(false, "");
				}
			});

			guiWnd.addActionListenerBtnFertigStellen(new ActionListener() {
				@Override
				public void actionPerformed(final ActionEvent e) {
					if (!guiWnd.ueberprufeEinstellungen(listLZZ)) {
						return;
					}

					final Object[] optionen1 = { "Abbrechen" };
					final Object[] optionen2 = { "OK" };

					final JOptionPane msg = new JOptionPane("Export läuft", JOptionPane.WARNING_MESSAGE,
							JOptionPane.OK_OPTION, null, optionen1, optionen1[0]);
					final JDialog dialog = new JDialog(guiWnd, "BAStBand", true);

					dialog.setContentPane(msg);
					dialog.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
					dialog.pack();
					dialog.setLocationRelativeTo(guiWnd);

					msg.addPropertyChangeListener(new PropertyChangeListener() {
						@Override
						public void propertyChange(final PropertyChangeEvent e) {
							final String prop = e.getPropertyName();

							if (dialog.isVisible() && (e.getSource() == msg)
									&& (prop.equals(JOptionPane.VALUE_PROPERTY))) {
								dialog.setVisible(false);
							}
						}
					});

					ausgabeVerzeichnis = new File(guiWnd.getAusgabePfad());
					calendarStart.set(Calendar.YEAR, guiWnd.getAnfangJahr());
					calendarStart.set(Calendar.MONTH, guiWnd.getAnfangMonat());

					calendarEnd.set(Calendar.YEAR, guiWnd.getEndeJahr());
					calendarEnd.set(Calendar.MONTH, guiWnd.getEndeMonat());

					BAStFormat = BastFormat.getIntVonName(guiWnd.getBastFormat());
					ueberschreiben = guiWnd.getUeberSchreiben();

					abbrechen = false;

					final Thread t = new Thread(new Exporter(guiWnd, dialog));
					t.start();
					dialog.setVisible(true);

					if (msg.getValue().equals(optionen1[0])) {
						abbrechen = true;
						msg.setMessage("Prozess wird gleich gestoppt");
						msg.setOptions(optionen2);
						msg.setMessageType(JOptionPane.INFORMATION_MESSAGE);
						dialog.setVisible(true);
					}
				}
			});

			// guiWnd.setAlwaysOnTop(true);
			guiWnd.setVisible(true);

		} else {
			try {
				System.out.println("Export läuft");
				export();
				System.out.println("Export beendet");
			} catch (final Exception e) {
				BastBand.LOGGER.error("Fehler beim Export : " + e.getMessage());
			}
			disconnect(false, "");
		}
	}

	/**
	 * Sortiert die EingabePids nach dem ob sie LZZ od KB sind
	 */
	public void bearbeiteEingabeObjektePids() {
		for (final String pid : listLZZundKBparameter) {
			final SystemObject so = oClientDavConnection.getDataModel().getObject(pid);
			if (so == null) {
				continue;
			}

			if (so.getType().getPid().equals(BastBand.TYP_KONFIGURATIONSBEREICH)) {
				final Collection<String> lzz = getLzzVonKB(pid);
				if ((lzz != null) && !lzz.isEmpty()) {
					listLZZ.addAll(lzz);
					listKB.add(pid);
				}
			} else if (so.getType().getPid().equals(BastBand.TYP_LANGZEITZAEHLSTELLE)) {
				listLZZ.add(pid);
			} else {
				System.out.println("Objekt " + pid + " ist kein Konfigurationsbereich oder Langzeitzählstelle");
				disconnect(false, "");
				System.exit(0);
			}
		}
	}

	/**
	 * Liest aus der Konfiguration alle zugehoerige LZZ zu einen KB
	 *
	 * @param pidKB
	 *            KonfigurationsBereich
	 * @return Menge der LZZ
	 */
	public Collection<String> getLzzVonKB(final String pidKB) {

		final DataModel dm = oClientDavConnection.getDataModel();
		final LinkedList<SystemObjectType> types = new LinkedList<>();
		types.add(dm.getType(BastBand.TYP_LANGZEITZAEHLSTELLE));

		final LinkedList<ConfigurationArea> kbListe = new LinkedList<>();
		kbListe.add(dm.getObject(pidKB).getConfigurationArea());

		final Collection<SystemObject> lzzListe = oClientDavConnection.getDataModel().getObjects(kbListe, types,
				ObjectTimeSpecification.valid());

		final Collection<String> stringLzzListe = new LinkedList<>();

		for (final SystemObject so : lzzListe) {
			stringLzzListe.add(so.getPid());
		}
		return stringLzzListe;
	}

	/**
	 * Generiert ein Baum aus LZZ und KB, wo LZZ die einzelne Blaetter, der Baum
	 * kann in ein swing TreeView eingelegt werden, beim jeden Knoten ist ein
	 * Kontrollkaestchen, mit dem man dem ganzen Unterbaum auswaehlen kann
	 *
	 * @return Das Baum
	 */
	public DefaultMutableTreeNode generiereLZZBaum() {

		final DataModel dm = oClientDavConnection.getDataModel();
		final LinkedList<SystemObjectType> types = new LinkedList<>();
		types.add(dm.getType(BastBand.TYP_KONFIGURATIONSBEREICH));

		final Collection<SystemObject> kbList = oClientDavConnection.getDataModel().getObjects(null, types,
				ObjectTimeSpecification.valid());

		types.clear();
		types.add(dm.getType(BastBand.TYP_LANGZEITZAEHLSTELLE));

		final KontrollKaestchenKnoten hauptKnoten = new KontrollKaestchenKnoten(
				"Alle Konfigruationsbereiche mit Langzeitzählstellen");
		hauptKnoten.setAllowsChildren(true);

		final LinkedList<ConfigurationArea> kbListe2 = new LinkedList<>();

		for (final SystemObject kb : kbList) {

			kbListe2.clear();
			kbListe2.add(kb.getConfigurationArea());
			final Collection<SystemObject> lzzListe = oClientDavConnection.getDataModel().getObjects(kbListe2, types,
					ObjectTimeSpecification.valid());

			if ((lzzListe != null) && !lzzListe.isEmpty()) {
				final KontrollKaestchenKnoten knoten = new KontrollKaestchenKnoten(kb.getPid());
				hauptKnoten.add(knoten);
				knoten.setAllowsChildren(true);

				for (final SystemObject lzz : lzzListe) {
					final KontrollKaestchenKnoten k2 = new KontrollKaestchenKnoten(lzz.getPid());
					k2.setAllowsChildren(false);
					knoten.add(k2);

					for (final String pid : listLZZ) {
						if (lzz.getPid().equals(pid)) {
							k2.setIstAusgewaehlt(true);
						}
					}
				}
				for (final String pid : listKB) {
					if (kb.getPid().equals(pid)) {
						knoten.setIstAusgewaehlt(true);
					}
				}
			}

		}
		return hauptKnoten;
	}

	/**
	 *
	 * Wird ausgeworfen, wenn die Verbindung mit dem DAV nicht moeglich ist
	 *
	 */
	private class ArchivNichtVerfuegbarexception extends Exception {
		private static final long serialVersionUID = 1L;

		public ArchivNichtVerfuegbarexception(final String s) {
			super(s);
		}
	}

	/**
	 * Exportiert die Daten
	 * 
	 * @throws Exception
	 *             der Export war nicht erfolgreich
	 *
	 * @throws ArchivNichtVerfuegbarexception
	 *             Wenn der Archiv nicht verfuegbar ist
	 */
	public void export() throws Exception {
		try {
			bastExporter = new BastExporter(this.BAStFormat);
			bastExporter
					.setAspect(oClientDavConnection.getDataModel().getAspect(DUAKonstanten.ASP_PL_PRUEFUNG_LOGISCH));
			final ArchiveRequestManager archiv = oClientDavConnection.getArchive();
			if (!archiv.isArchiveAvailable()) {
				throw new ArchivNichtVerfuegbarexception("Archiv nicht verfügbar");
			}

		} catch (final Exception e) {
			if (e instanceof ArchivNichtVerfuegbarexception) {
				nachrichtenSender.sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.INFORMATION,
						"Fehler beim Zugriff zum Archiv: " + e.getMessage());
				throw new ArchivNichtVerfuegbarexception("Fehler beim Zugriff auf Archiv: " + e.getMessage());
			}
			throw new Exception("Fehler bei Erstellung des Exporters: " + e.getMessage());
		}
		for (final String lzzPid : listLZZ) {

			final Data oData = oClientDavConnection.getDataModel().getObject(lzzPid)
					.getConfigurationData(oClientDavConnection.getDataModel().getAttributeGroup(BastBand.ATG_LZZ));

			if (oData == null) {
				BastBand.LOGGER.error("Langzeitzählstelle " + lzzPid + " nicht konfiguriert");
				continue;
			}
			final LangZeitZaehlStelle lzz = LangZeitZaehlStelle.getInstance(lzzPid);
			final Calendar iterator = Calendar.getInstance();
			iterator.setTimeInMillis(calendarStart.getTimeInMillis());

			if (lzz == null) {
				BastBand.LOGGER.error("Langzeitzählstelle " + lzzPid + " nicht gefunden");
			} else {
				do {
					if (abbrechen) {
						BastBand.LOGGER.warning("Export abgebrochen von Benutzer");
						return;
					}
					try {
						if (this.BAStFormat == BastExporter.BAST_FORMAT_AXL) {
							bastExporter.export(lzz, ausgabeVerzeichnis, iterator.get(Calendar.YEAR),
									iterator.get(Calendar.MONTH));
						} else {
							File file;
							if (ueberschreiben) {
								file = generiereLetzteFile(lzz, ausgabeVerzeichnis, iterator.get(Calendar.YEAR),
										iterator.get(Calendar.MONTH));
							} else {
								file = generiereNaechsteFile(lzz, ausgabeVerzeichnis, iterator.get(Calendar.YEAR),
										iterator.get(Calendar.MONTH));
							}
							bastExporter.export(lzz, file, iterator.get(Calendar.YEAR), iterator.get(Calendar.MONTH));
						}
						nachrichtenSender.sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.INFORMATION,
								"Daten für LZZ " + lzzPid + " Zeitbereich: "
										+ BastBandGUI.MONATE[iterator.get(Calendar.MONTH)] + " "
										+ iterator.get(Calendar.YEAR) + " exportiert.");
					} catch (final Exception e) {
						e.printStackTrace();
						nachrichtenSender.sendMessage(MessageType.APPLICATION_DOMAIN, MessageGrade.INFORMATION,
								"Fehler bei Erstellung einer Datei: " + lzzPid + " Ursache: " + e.getMessage());
						throw new Exception(
								"Fehler bei Erstellung einer Datei: " + lzzPid + " Ursache:" + e.getMessage());
					}
					iterator.add(Calendar.MONTH, 1);
				} while (iterator.getTimeInMillis() <= calendarEnd.getTimeInMillis());
			}
		}
	}

	/**
	 * Trennt die Verbindung
	 *
	 * @param err
	 *            true wenn man mit Fehler beendet hat
	 * @param msg
	 *            Fehlernachricht
	 */
	public void disconnect(final boolean err, final String msg) {
		// weil WindowCloseEvent kann mehrmal kommen
		if ((oClientDavConnection != null) && verbunden) {
			oClientDavConnection.disconnect(err, msg);
			verbunden = false;
		}
	}

	/**
	 * Anwendungsparameter
	 */
	public static void printVerwendung() {
		System.out.println("Ergänzung BASt-Band:\n");
		System.out.println("Unterstützte Parameter: ");
		System.out.println("\t" + BastBand.PAR_GUIBASTBAND);
		System.out.println("\t" + BastBand.PAR_ZEITBEREICH + "=monatVon.jahrVon[:monatBis.jahrBis] im Format MM.YYYY");
		System.out.println("\t" + BastBand.PAR_BASTFORMAT + "=V2004|SA|MV|AXL|MVTest");
		System.out.println("\t" + BastBand.PAR_AUSGABEGERAET + "=Pfad zum Verzeichnis");
		System.out.println("\t" + BastBand.PAR_KONFBERIECHPID + "=pid(,pid)");
		System.out.println("\t" + BastBand.PAR_UEBERSCHREIBEN + "=true|false");
	}

	/**
	 * Eine Statische Klasse fuer die Konvertierung von Namen, Int Konstanten und
	 * Kommandozeile-Parameter der BastBand Formate
	 *
	 */
	static private class BastFormat {
		private static List<BastFormat> bastFormate = new ArrayList<>();

		static {
			BastFormat.bastFormate.add(new BastFormat("MV", BastExporter.BAST_FORMAT_MV, "Mecklenburg-Vorpommern"));
			BastFormat.bastFormate.add(new BastFormat("SA", BastExporter.BAST_FORMAT_SA, "Sachsen-Anhalt"));
			BastFormat.bastFormate.add(new BastFormat("MVTest", BastExporter.BAST_FORMAT_MV_TEST, "Test MV"));
			BastFormat.bastFormate.add(new BastFormat("V2004", BastExporter.BAST_FORMAT_V2004, "Version 2004"));
			BastFormat.bastFormate.add(new BastFormat("AXL", BastExporter.BAST_FORMAT_AXL, "Achslastdaten"));
		}

		private final String kommando;
		private final String name;
		private final int value;

		BastFormat(final String komm, final int v, final String name) {
			this.kommando = komm;
			this.name = name;
			this.value = v;
		}

		static String getNameVonInt(final int v) {
			for (final BastFormat bf : BastFormat.bastFormate) {
				if (bf.value == v) {
					return bf.name;
				}
			}
			return null;
		}

		static int getIntVonKommandoParam(final String kommando) {
			for (final BastFormat bf : BastFormat.bastFormate) {
				if (bf.kommando.equals(kommando)) {
					return bf.value;
				}
			}
			return -1;
		}

		static int getIntVonName(final String name) {
			for (final BastFormat bf : BastFormat.bastFormate) {
				if (bf.name.equals(name)) {
					return bf.value;
				}
			}
			return -1;
		}
	}

	/**
	 *
	 * Wird als working-therad benutzt. Ermoeglicht, dass durch das GUI man dem
	 * Export abbrechen kann.
	 *
	 */
	private class Exporter implements Runnable {

		JDialog mainWnd;
		JDialog wnd;

		/**
		 * Konstruktor
		 *
		 * @param mainWnd
		 *            HauptFenster der Applikation
		 * @param wnd
		 *            Fenster mit der Meldung "Export Lauft"
		 */
		Exporter(final JDialog mainWnd, final JDialog wnd) {
			this.wnd = wnd;
			this.mainWnd = mainWnd;
		}

		/**
		 * Arbeitermethode
		 */
		@Override
		public void run() {

			try {
				export();
			} catch (final Exception e) {
				JOptionPane.showMessageDialog(mainWnd, e.getMessage(), "BAStExporter", JOptionPane.ERROR_MESSAGE);
			}
			if ((wnd != null) && (abbrechen == false)) {
				wnd.setVisible(false);
				JOptionPane.showMessageDialog(mainWnd, "Export beendet", "BAStExporter",
						JOptionPane.INFORMATION_MESSAGE);
			} else if (wnd != null) {
				wnd.setVisible(false);
				JOptionPane.showMessageDialog(mainWnd, "Export wurde abgebrochen", "BAStExporter",
						JOptionPane.INFORMATION_MESSAGE);
			}
		}
	}

	/**
	 *
	 * Generiert den BASt Filename aus dem Namen der LZZ
	 *
	 * @param lzz
	 *            LangzeitZaehlstelle
	 * @param Jahr
	 *            Der exportierte Jahr
	 * @param Monat
	 *            Der exportiert Monat
	 * @return Filename
	 */
	public String generiereFileName(final LangZeitZaehlStelle lzz, final int Jahr, final int Monat) {

		final String name = lzz.getBundesland().getAbbreviation() + lzz.getBastNummerAsString();
		final String ext = String.format(".%02d%h", Jahr % 100, Monat + 1);
		return name + ext;
	}

	/**
	 *
	 * Generiert den BASt Filename der noch existiert nicht in gegebenen
	 * Verzeichniss
	 *
	 * @param lzz
	 *            LangzeitZaehlstelle
	 * @param dir
	 *            Verzeichniss
	 * @param Jahr
	 *            Der exportierte Jahr
	 * @param Monat
	 *            Der exportierte Monat
	 * @return File in dem die Daten exportiert werden sollen
	 */
	public File generiereNaechsteFile(final LangZeitZaehlStelle lzz, final File dir, final int Jahr, final int Monat) {

		final String ext = String.format(".%02d%h", Jahr % 100, Monat + 1);

		final StringBuffer sb = new StringBuffer(dir.getAbsolutePath());
		sb.append(File.separatorChar);
		sb.append(lzz.getBundesland().getAbbreviation());
		sb.append(lzz.getBastNummerAsString());

		final String pathName = sb.toString();

		sb.append(ext);
		File f = new File(sb.toString());
		if (!f.exists()) {
			return f;
		}

		int i = 1;
		do {
			sb.setLength(0);
			sb.append(pathName);
			sb.append(String.format("_%d", i++));
			sb.append(ext);
			f = new File(sb.toString());
		} while (f.exists());

		return f;
	}

	/**
	 * Generiert den BASt Filename der entweder noch nicht existiert in gegebenen
	 * Verzeichniss, als letzte Version ueberschrieben wird
	 *
	 * @param lzz
	 *            LangzeitZaehlstelle
	 * @param dir
	 *            Verzeichniss
	 * @param Jahr
	 *            Der exportierte Jahr
	 * @param Monat
	 *            Der exportierte Monat
	 * @return File in dem die Daten exportiert werden sollen
	 */
	public File generiereLetzteFile(final LangZeitZaehlStelle lzz, final File dir, final int Jahr, final int Monat) {

		final String ext = String.format(".%02d%h", Jahr % 100, Monat + 1);

		final StringBuffer sb = new StringBuffer(dir.getAbsolutePath());
		sb.append(File.separatorChar);
		sb.append(lzz.getBundesland().getAbbreviation());
		sb.append(lzz.getBastNummerAsString());

		final String pathName = sb.toString();

		sb.append(ext);
		File alt_f, f = new File(sb.toString());
		if (!f.exists()) {
			return f;
		}

		int i = 1;
		do {
			sb.setLength(0);
			sb.append(pathName);
			sb.append(String.format("_%d", i++));
			sb.append(ext);
			alt_f = f;
			f = new File(sb.toString());
		} while (f.exists());
		return alt_f;
	}
}
