/**
 * Segment 4 Datenübernahme und Aufbereitung (DUA), SWE 4.10 Ergänzung BASt-Band
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.bastband.gui;

import java.util.Collection;

import javax.swing.tree.DefaultMutableTreeNode;

/**
 * Ein Knoten des Baumes im JTreeView, kann seinen Unterbaum auch auswählen,
 * oder kann nur einfache Auswahl ermöglichen
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class KontrollKaestchenKnoten extends DefaultMutableTreeNode {

	/**
	 * Autogenerated
	 */
	private static final long serialVersionUID = 5005027249482529499L;
	/**
	 * Methoden der Auswahl - einfach oder dem ganzen Unterbaum
	 */
	public final static int SINGLE_AUSWAHL = 0;
	public final static int UNTERBAUM_AUSWAHL = 4;

	/**
	 * Konkrete Methode dieses Knotens
	 */
	protected int auswahlWeise;

	/**
	 * Methode der Auswahl - einfach oder dem ganzen Unterbaum
	 */
	protected boolean ausgewaehlt;

	/**
	 * Verschiedene Kontstruktore
	 */
	public KontrollKaestchenKnoten() {
		this(null);
	}

	/**
	 * Konstruktor
	 *
	 * @param benutzerObjekt
	 *            Der Objekt dem das HauptKnoten enthalten wird
	 */
	public KontrollKaestchenKnoten(final Object benutzerObjekt) {
		this(benutzerObjekt, true, false);
	}

	/**
	 * Konstruktor
	 *
	 * @param benutzerObjekt
	 *            Der Objekt dem das HauptKnoten enthalten wird
	 * @param hatSohne
	 *            true, wenn der HauptKnoten Sohne hat
	 * @param ausgewaehlt
	 *            true, wenn der Hauptknoten ausgewaehlt ist
	 */
	public KontrollKaestchenKnoten(final Object benutzerObjekt, final boolean hatSohne, final boolean ausgewaehlt) {
		super(benutzerObjekt, hatSohne);
		this.ausgewaehlt = ausgewaehlt;
		setAuswahlWeise(KontrollKaestchenKnoten.UNTERBAUM_AUSWAHL);
	}

	/**
	 * Stellt fest die Auswahlweise ein
	 *
	 * @param auswahlWeise
	 *            Die Auswahlweise (Single / Unterbaum)
	 */
	public void setAuswahlWeise(final int auswahlWeise) {
		this.auswahlWeise = auswahlWeise;
	}

	/**
	 * Erfragt die Auswahlweise (Single / Unterbaum)
	 *
	 * @return Die Auswahlweise
	 */
	public int getAuswahlWeise() {
		return auswahlWeise;
	}

	/**
	 * Stellt fest dem Knoten dem ausgewaehlt Paramter ein geht rekursiv in den
	 * Unterbaum
	 *
	 * @param istAusgewaehlt
	 *            true, wenn es ausgewaehlt ist
	 */
	public void setIstAusgewaehlt(final boolean istAusgewaehlt) {
		this.ausgewaehlt = istAusgewaehlt;

		if ((auswahlWeise == KontrollKaestchenKnoten.UNTERBAUM_AUSWAHL) && (children != null)) {
			for (int i = 0; i < children.size(); i++) {
				final KontrollKaestchenKnoten knoten = (KontrollKaestchenKnoten) children.get(i);
				knoten.setIstAusgewaehlt(istAusgewaehlt);
			}
		}
	}

	/**
	 * Schreibt die ausgewaehlten Endknoten (Blaetter des Baums) in die Liste
	 *
	 * @param selectedItems
	 *            die Liste, die aktualsiert werden soll
	 */
	public void updateSelections(final Collection<String> selectedItems) {
		final String name = (String) this.getUserObject();

		if (!allowsChildren) {
			if (ausgewaehlt && !selectedItems.contains(name)) {
				selectedItems.add(name);
			} else if (!ausgewaehlt && selectedItems.contains(name)) {
				selectedItems.remove(name);
			}
		} else if (children != null) {
			for (int i = 0; i < children.size(); i++) {
				final KontrollKaestchenKnoten knoten = (KontrollKaestchenKnoten) children.get(i);
				knoten.updateSelections(selectedItems);
			}
		}
	}

	/**
	 * Erfragt ob der Knoten Ausgewaehlt ist
	 *
	 * @return true, wenn ausgewaehlt
	 */
	public boolean isAusgewaehlt() {
		return ausgewaehlt;
	}
}
